module Knuckleheads

  require_relative '../knuckleheads/bill_player'
  require_relative '../knuckleheads/clumsy_player'
  require_relative '../knuckleheads/crazy_player'
  require_relative '../knuckleheads/game_turn'
  require_relative '../knuckleheads/hilary_player'
  require_relative '../knuckleheads/homo_nazi_player'
  require_relative '../knuckleheads/islamic_nazi_player'
  require_relative '../knuckleheads/player'
  require_relative '../knuckleheads/treasure_trove'

  class Game

    attr_reader :title

    def initialize(title)
      @title = title
      @players = []
    end

    def add_player(a_player)
      @players << a_player
    end

    def play(rounds)
      puts "There are #{@players.size} players in #{@title}: "
      @players.each do |player|
        puts player
      end
      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures to be found:"
      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points"
      end

      1.upto(rounds) do |round|
         puts "\nRound #{round}:"
        @players.each do |player|
          GameTurn.take_turn(player)
        end
      end
    end

    def print_high_scores
      puts "\n#{@title} High Scores:"
      @players.sort.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.health} health points"
        puts "#{player.score} grand total score"
      end    
    end

    def load_players(from_file)
      File.readlines(from_file).each do |line|
        line = line.chomp
        name, type = line.split(",")
        case type
        when 'bill'
          player = BillPlayer.new(name)
        when "clumsy"
          player = ClumsyPlayer.new(name)
        when "crazy"
          player = CrazyPlayer.new(name)
        when 'hilary'
          player = HilaryPlayer.new(name)
        when 'homonazi'
          player = HomoNaziPlayer.new(name)
        when 'islamicnazi'
          player = IslamicNaziPlayer.new(name)
        else
          player = Player.new(name)
        end
        add_player(player)
      end
    end

    def save_high_scores(to_file="high_scores.txt")
      File.open(to_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        place = 0
        @players.sort.each do |player|
          place += 1
          formatted_name = "#{place}. #{player.name}".ljust(20, '.')
          formatted_place = player.score.to_s.rjust(10,'.')
          file.puts "#{formatted_name}#{formatted_place}"
        end
      end
    end

  end

end
